#include <stdint.h>
#include "stc8h.h"

/* CONFIGURATIONS */
#define LCD_WIDTH       128u
#define LCD_HEIGHT      160u
#define LCD_OFFSET_X    2u
#define LCD_OFFSET_Y    1u

/**
 * CONNECTION: 
 * CS   -   P1.1
 * SDA  -   P1.3/MOSI
 * SCL  -   P1.5/SCLK
 * RST  -   P1.6
 * DC   -   P1.7
 */

void lcdCmdTx(uint8_t);
void lcdDatTx(uint8_t);
void lcdD16Tx(uint16_t);
void lcdInit(void);
void lcdFill(uint8_t, uint8_t, uint8_t, uint8_t, uint16_t);

void delay_ms(uint32_t ms)
{
    for (uint32_t i = 0; i < ms; i++)
    {
        for (uint16_t j = 0; j < 1000; j++);
    }
}

int main(void)
{
    // IO init
    P1M0 = 0x00;
    P1M1 = 0x00;

    // SPI init
    SPCTL = 0xD2;
    SPSTAT = 0xC0;

    P11 = 1;

    // LCD init and display
    lcdInit();

    lcdFill(0u,     0u, 127u,   159u,   0xF81Fu);
    lcdFill(0u,     0u, 7u,     7u,     0xFFFFu);
    lcdFill(8u,     0u, 15u,    7u,     0x7D7Cu);
    lcdFill(0u,     8u, 7u,     15u,    0xA651u);
    lcdFill(8u,     8u, 15u,    15u,    0x0000u);
    lcdFill(16u,    0u, 31u,    15u,    0x001Fu);
    lcdFill(32u,    0u, 47u,    15u,    0xF81Fu);
    lcdFill(48u,    0u, 63u,    15u,    0xFFE0u);
    lcdFill(64u,    0u, 79u,    15u,    0x07FFu);
    lcdFill(80u,    0u, 95u,    15u,    0xF800u);
    lcdFill(96u,    0u, 111u,   15u,    0xFFE0u);
    lcdFill(112u,   0u, 127u,   15u,    0x8430u);

    while (1);
}

/**
 * @brief transmit 1-byte command via SPI
 * 
 * @param command command to be transmitted
 */
void lcdCmdTx(uint8_t command)
{
    P11 = 0;
    P17 = 0;

    SPDAT = command;
    while (!(SPSTAT & 0x80));
    SPSTAT = 0xC0;

    P11 = 1;
}

/**
 * @brief transmit 1-byte data via SPI
 * 
 * @param data data to be transmitted
 */
void lcdDatTx(uint8_t data)
{
    P11 = 0;
    P17 = 1;

    SPDAT = data;
    while (!(SPSTAT & 0x80));
    SPSTAT = 0xC0;

    P11 = 1;
}

/**
 * @brief transmit a 2-byte (16-bit) data with MSB first via SPI
 * 
 * @param data data to be transmitted
 */
void lcdD16Tx(uint16_t data)
{
    P11 = 0;
    P17 = 1;

    SPDAT = (data & 0xFF00) >> 8;
    while (!(SPSTAT & 0x80));
    SPSTAT = 0xC0;

    SPDAT = (data & 0x00FF);
    while (!(SPSTAT & 0x80));
    SPSTAT = 0xC0;

    P11 = 1;
}

/**
 * @brief initialize the LCD display 
 */
void lcdInit(void)
{
    // RESET
    P16 = 1;
    delay_ms(50);
    P16 = 0;
    delay_ms(50);
    P16 = 1;
    delay_ms(50);

    lcdCmdTx(0x11);//Sleep exit 
    delay_ms(120);
    
    //ST7735R Frame Rate
    lcdCmdTx(0xB1); 
    lcdDatTx(0x01); 
    lcdDatTx(0x2C); 
    lcdDatTx(0x2D); 
    lcdCmdTx(0xB2); 
    lcdDatTx(0x01); 
    lcdDatTx(0x2C); 
    lcdDatTx(0x2D); 
    lcdCmdTx(0xB3); 
    lcdDatTx(0x01); 
    lcdDatTx(0x2C); 
    lcdDatTx(0x2D); 
    lcdDatTx(0x01); 
    lcdDatTx(0x2C); 
    lcdDatTx(0x2D); 	
    lcdCmdTx(0xB4); //Column inversion 
    lcdDatTx(0x07); 	
    //ST7735R Power Sequence
    lcdCmdTx(0xC0); 
    lcdDatTx(0xA2); 
    lcdDatTx(0x02); 
    lcdDatTx(0x84); 
    lcdCmdTx(0xC1); 
    lcdDatTx(0xC5); 
    lcdCmdTx(0xC2); 
    lcdDatTx(0x0A); 
    lcdDatTx(0x00); 
    lcdCmdTx(0xC3); 
    lcdDatTx(0x8A); 
    lcdDatTx(0x2A); 
    lcdCmdTx(0xC4); 
    lcdDatTx(0x8A); 
    lcdDatTx(0xEE); 
    lcdCmdTx(0xC5); //VCOM 
    lcdDatTx(0x0E); 	
    lcdCmdTx(0x36); //MX, MY, RGB mode 
    lcdDatTx(0xC0); 
    //ST7735R Gamma Sequence
    lcdCmdTx(0xe0); 
    lcdDatTx(0x0f); 
    lcdDatTx(0x1a); 
    lcdDatTx(0x0f); 
    lcdDatTx(0x18); 
    lcdDatTx(0x2f); 
    lcdDatTx(0x28); 
    lcdDatTx(0x20); 
    lcdDatTx(0x22); 
    lcdDatTx(0x1f); 
    lcdDatTx(0x1b); 
    lcdDatTx(0x23); 
    lcdDatTx(0x37); 
    lcdDatTx(0x00); 	
    lcdDatTx(0x07); 
    lcdDatTx(0x02); 
    lcdDatTx(0x10); 
    lcdCmdTx(0xe1); 
    lcdDatTx(0x0f); 
    lcdDatTx(0x1b); 
    lcdDatTx(0x0f); 
    lcdDatTx(0x17); 
    lcdDatTx(0x33); 
    lcdDatTx(0x2c); 
    lcdDatTx(0x29); 
    lcdDatTx(0x2e); 
    lcdDatTx(0x30); 
    lcdDatTx(0x30); 
    lcdDatTx(0x39); 
    lcdDatTx(0x3f); 
    lcdDatTx(0x00); 
    lcdDatTx(0x07); 
    lcdDatTx(0x03); 
    lcdDatTx(0x10);  	
    lcdCmdTx(0x2a);
    lcdDatTx(0x00);
    lcdDatTx(0x00);
    lcdDatTx(0x00);
    lcdDatTx(0x7f);
    lcdCmdTx(0x2b);
    lcdDatTx(0x00);
    lcdDatTx(0x00);
    lcdDatTx(0x00);
    lcdDatTx(0x9f);
    lcdCmdTx(0xF0); //Enable test command  
    lcdDatTx(0x01); 
    lcdCmdTx(0xF6); //Disable ram power save mode 
    lcdDatTx(0x00); 	
    lcdCmdTx(0x3A); //65k mode 
    lcdDatTx(0x05); 
    lcdCmdTx(0x29); //display on	

    // horizontal
    lcdCmdTx(0x36);
    lcdDatTx(0xC0); //BGR==1,MY==0,MX==0,MV==0
}

/**
 * @brief fill a square area with specific color
 * 
 * @param sx x of start point, or the left top point
 * @param sy y of start point, or the left top point
 * @param ex x of end point, or the right bottom point
 * @param ey y of end point, or the right bottom point
 * @param color the color will be filled in RGB565 (RRRRRGGG_GGGBBBBB)
 */
void lcdFill(uint8_t sx, uint8_t sy, uint8_t ex, uint8_t ey, uint16_t color)
{
    lcdCmdTx(0x2Au);	
    lcdDatTx(0x00u);
    lcdDatTx(sx+LCD_OFFSET_X);		
    lcdDatTx(0x00u);
    lcdDatTx(ex+LCD_OFFSET_X);

    lcdCmdTx(0x2Bu);	
    lcdDatTx(0x00u);
    lcdDatTx(sy+LCD_OFFSET_Y);
    lcdDatTx(0x00u);
    lcdDatTx(ey+LCD_OFFSET_Y);

    lcdCmdTx(0x2Cu);
    for (uint16_t y = sy; y <= ey; y++)
    {
        for (uint16_t x = sx; x <= ex; x++)
        {
            lcdD16Tx(color);
        }
    }

    lcdCmdTx(0x2Au);	
    lcdDatTx(0x00u);
    lcdDatTx(0u+LCD_OFFSET_X);		
    lcdDatTx(0x00u);
    lcdDatTx(LCD_WIDTH-1u+LCD_OFFSET_X);

    lcdCmdTx(0x2Bu);	
    lcdDatTx(0x00u);
    lcdDatTx(0u+LCD_OFFSET_Y);		
    lcdDatTx(0x00u);
    lcdDatTx(LCD_HEIGHT-1u+LCD_OFFSET_Y);

    lcdCmdTx(0x2Cu);
}
